# Frontend settings manager

The `frontend settings manager` stores various settings and some data in the client's browser.

Make sure you don't save any crucial in the manager because the data is stored in `local storage` in the browser

Thus the user (or a browser update maybe?) can clear the data also the size of the local storage size limit is different in the various browsers and some may not support `local storage` at all

The `frontend settings manager` is found at `frontend/src/helpers/frontendSettingsManager.ts`

The data that is stored is

- `view states` for sites (table vs card layout)
- `sort directions` for sites
- `last logged in user id`
	- we need to clear the user data (and all other data) if we get a login timeout and another user logs in
- some global settings for some components
- `last user solution` if the browser crashes we want to try to recover the last known solution


Because the data can change quickly e.g. in case of the do exercise editor every keystroke leads to a state change (a new user solution) we throttle the `frontend settings manager` so that at most only every X seconds the data in the `frontend settings manager` can change


*There are some tweaks that can only be explained in the next section*

## Adding new settings or data

This is pretty simple because we only need to take care of 2 files.

**The first one** is the `frontend settings manager` itself so go ahead and add the setting or data as a new property.
Saving and loading is done via json serialization so nothing to do here.

Then add a function to the `frontend settings manager` to set the property value (we go the redux style) and check in this function if the state manager is frozen.

This is as easy as

```ts
static setMyValueState(value: MyType): void {

  if (FrontendSettingsManager.getIsFrozen()) return

  frontendSettings.value = value
  saveFrontendSettingsThrottled()
}
```

*Why do we need to check this?*

Well read ahead. It has something to do with the way we implement setting & reading the settings.


**The second file** we need to edit is the reducer that holds the data for the component we want to load/save data for

```ts
export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {
    case ActionType.SET_value:

      FrontendSettingsManager.setMyValueState(action.value)

      return {
        ...state,
        value: action.value
      }

    case ActionType.RESET:
      return {
        ...initial,
        value: FrontendSettingsManager.getFrontendSettings().myValue,
      }
  }
}
```

So every time we change the value with the reducer we automatically update the value inside the `frontend settings manager`.

Whenever we call reset which should be called before any site is rendered (to clear old data) we overwrite the initial value of `value` with the value from the `frontend settings manager`.

`getFrontendSettings()` is a function ensuring we read the settings from the `local storage`


The only problem left is that we sometimes don't want to overwrite the values in the `frontend settings manager`.

e.g. when the do exercise site is loading the settings were already read and we are about to receive the user solution

In this scenario we first clear any old user solution in the do exercise site reducer which would overwrite the read user solution from the `local storage`
and then we receive the user solution from backend which then overwrites the *initial solution* with the solution from backend.

In fact we want to keep the *old* user solution from `local storage` as long as possible. To achieve this we freeze (or lock) the `frontend settings manager` and save the *old* user solution from being overwritten. We don't need to set the backend solution as the *old* user solution because it's saved at backend.

After everything is set and the user starts changing the solution we overwrite the *old* user solution in the `frontend settings manager`.

